@@ -18,11 +18,6 @@
  USA
 **/
 
-
-/*
- * kig.cpp
- *
- */
 #include "kig.h"
 #include "kig.moc"
 
@@ -42,6 +37,8 @@
 #include <klocale.h>
 #include <kimageio.h>
 
+#include <assert.h>
+
 Kig::Kig()
   : KParts::MainWindow( 0L, "Kig" )
 {
@@ -77,7 +74,6 @@ Kig::Kig()
       KApplication::exit();
       return;
   }
-  resize (640,480);
 
   // we have drag'n'drop
   setAcceptDrops(true);
@@ -96,8 +92,13 @@ void Kig::setupActions()
   KStdAction::open(this, SLOT(fileOpen()), actionCollection());
   KStdAction::quit(this, SLOT(close()), actionCollection());
 
+#ifdef KIG_DONT_USE_NEW_KMAINWINDOW_FEATURES
   m_toolbarAction = KStdAction::showToolbar(this, SLOT(optionsShowToolbar()), actionCollection());
   m_statusbarAction = KStdAction::showStatusbar(this, SLOT(optionsShowStatusbar()), actionCollection());
+#else
+  createStandardStatusBarAction();
+  setStandardToolBarMenuEnabled(true);
+#endif
 
   // FIXME: this (recent files) should be app-wide, not specific to each window...
   m_recentFilesAction = KStdAction::openRecent(this, SLOT(openURL(const KURL&)), actionCollection());
@@ -161,22 +162,6 @@ void Kig::openURL(const KURL& url)
   };
 }
 
-void Kig::optionsShowToolbar()
-{
-  if (m_toolbarAction->isChecked())
-    toolBar()->show();
-  else
-    toolBar()->hide();
-}
-
-void Kig::optionsShowStatusbar()
-{
-  if (m_statusbarAction->isChecked())
-    statusBar()->show();
-  else
-    statusBar()->hide();
-}
-
 void Kig::optionsConfigureKeys()
 {
   KKeyDialog::configureKeys(actionCollection(), "kigui.rc");
@@ -243,7 +228,36 @@ void Kig::dropEvent(QDropEvent* e)
 void Kig::fileOpen()
 {
   // this slot is connected to the KStdAction::open action...
-  QString file_name = KFileDialog::getOpenFileName(":document", i18n("*.kig|Kig Documents (*.kig)\n*.kgeo|KGeo Documents (*.kgeo)"));
+  QString file_name = KFileDialog::getOpenFileName(":document", i18n("*.kig|Kig Documents (*.kig)\n"
+                                                                     "*.kgeo|KGeo Documents (*.kgeo)\n"
+                                                                     "*.kig *.kgeo|All Supported Files (*.kig *.kgeo)"));
 
   if (!file_name.isEmpty()) openURL(file_name);
 }
+
+// ifdef's disabled, cause Qt moc doesn't handle ifdef's..
+// #ifdef KIG_DONT_USE_NEW_KMAINWINDOW_FEATURES
+void Kig::optionsShowToolbar()
+{
+#ifdef KIG_DONT_USE_NEW_KMAINWINDOW_FEATURES
+  if (m_toolbarAction->isChecked())
+    toolBar()->show();
+  else
+    toolBar()->hide();
+#else
+  assert( false );
+#endif
+}
+
+void Kig::optionsShowStatusbar()
+{
+#ifdef KIG_DONT_USE_NEW_KMAINWINDOW_FEATURES
+  if (m_statusbarAction->isChecked())
+    statusBar()->show();
+  else
+    statusBar()->hide();
+#else
+  assert( false );
+#endif
+}
+// #endif
