@@ -22,9 +22,7 @@
 #include "kig_view.moc"
 
 #include "kig_part.h"
-#include "exporttoimagedialog.h"
 #include "../objects/object.h"
-#include "../objects/point.h"
 #include "../misc/coordinate_system.h"
 
 #include "../modes/mode.h"
@@ -67,7 +65,7 @@ KigWidget::KigWidget( KigDocument* doc,
   setBackgroundMode( Qt::NoBackground );
   setMouseTracking(true);
 
-  curPix.resize(size());
+  curPix.resize( size() );
   stillPix.resize( size() );
 };
 
@@ -85,12 +83,6 @@ void KigView::setupActions()
   aCenterScreen = KStdAction::fitToPage( mrealwidget, SLOT( recenterScreen() ),
                                          mdoc->actionCollection() );
   aCenterScreen->setWhatsThis( i18n( "Recenter the screen on the document" ) );
-
-  aExportToImage = new KAction( i18n( "Export to Image..." ), 0,
-                                this, SLOT( slotExportToImage() ),
-                                mdoc->actionCollection(), "file_export_to_image" );
-  aExportToImage->setWhatsThis( i18n( "This function allows you to export the currently "
-                                      "showing screen to an image file." ) );
 }
 
 KigWidget::~KigWidget()
@@ -161,12 +153,31 @@ void KigWidget::updateEntireWidget()
   updateWidget( overlay );
 }
 
-void KigWidget::resizeEvent(QResizeEvent*)
+void KigWidget::resizeEvent( QResizeEvent* e )
 {
-  recenterScreen();
-  curPix.resize(size());
-  stillPix.resize( size() );
+  QSize osize = e->oldSize();
+  QSize nsize = e->size();
+  Rect orect = msi.shownRect();
+
+  curPix.resize( nsize );
+  stillPix.resize( nsize );
   msi.setViewRect( rect() );
+
+  Rect nrect( 0., 0.,
+              orect.width() * nsize.width() / osize.width(),
+              orect.height() * nsize.height() / osize.height() );
+  nrect = matchScreenShape( nrect );
+  nrect.setTopLeft( orect.topLeft() );
+  msi.setShownRect( nrect );
+
+  // horrible hack...  We need to somehow differentiate between the
+  // resizeEvents we get on startup, and the ones generated by the
+  // user.  The first require recentering the screen, the latter
+  // don't..
+  if ( nsize.width() / osize.width() > 4 ) recenterScreen();
+
+  updateScrollBars();
+
   redrawScreen();
 }
 
@@ -185,16 +196,9 @@ void KigWidget::updateCurPix( const std::vector<QRect>& ol )
 
 void KigWidget::recenterScreen()
 {
-  msi.setShownRect( matchScreenShape(mdocument->suggestedRect()) );
-//   kdDebug() << k_funcinfo << endl
-//  // 	    << "(0,0): " << toScreen(Coordinate(0,0)) << endl
-// // 	    << "(-3,-2): " << toScreen( Coordinate(-3,-2)) << endl
-// // 	    << "(3,2): " << toScreen( Coordinate(3,2)) << endl;
-//  	    << "showingRect:" << showingRect() << endl
-//  	    << "fromScreen(...): " << fromScreen(QRect(QPoint(0,0), size()))
-//  	    << endl;
-  redrawScreen();
+  msi.setShownRect( matchScreenShape( mdocument->suggestedRect() ) );
   updateScrollBars();
+  redrawScreen();
 }
 
 Rect KigWidget::matchScreenShape( const Rect& r ) const
@@ -255,7 +259,7 @@ void KigWidget::redrawScreen( bool dos )
 {
   // update the screen...
   clearStillPix();
-  KigPainter p( msi, &stillPix );
+  KigPainter p( msi, &stillPix, *mdocument );
   p.drawGrid( mdocument->coordinateSystem() );
   p.drawObjects( mdocument->objects() );
   updateCurPix( p.overlay() );
@@ -267,7 +271,7 @@ const ScreenInfo& KigWidget::screenInfo() const
   return msi;
 }
 
-const Rect KigWidget::showingRect()
+const Rect KigWidget::showingRect() const
 {
   return msi.shownRect();
 }
@@ -323,8 +327,8 @@ KigView::KigView( KigDocument* doc,
 
   setupActions();
 
+  resize( sizeHint() );
   mrealwidget->recenterScreen();
-  mrealwidget->redrawScreen( false );
 }
 
 void KigView::updateScrollBars()
@@ -354,9 +358,9 @@ void KigView::updateScrollBars()
   double pw = mrealwidget->screenInfo().pixelWidth();
 
   // what the scrollbars reflect is the bottom resp. the left side of
-  // the shown rect.  This is why the maximum value is not ertop
+  // the shown rect.  This is why the maximum value is not er.top()
   // (which would be the maximum value of the top of the shownRect),
-  // but ertop - sr.height(), which is the maximum value the bottom of
+  // but er.top() - sr.height(), which is the maximum value the bottom of
   // the shownRect can reach...
 
   int rightmin = static_cast<int>( er.bottom() / pw );
@@ -364,21 +368,21 @@ void KigView::updateScrollBars()
 
   mrightscroll->setMinValue( rightmin );
   mrightscroll->setMaxValue( rightmax );
-  mrightscroll->setLineStep( sr.height() / pw / 10 );
-  mrightscroll->setPageStep( sr.height() / pw / 1.2 );
+  mrightscroll->setLineStep( (int)( sr.height() / pw / 10 ) );
+  mrightscroll->setPageStep( (int)( sr.height() / pw / 1.2 ) );
 
   // note that since Qt has a coordinate system with the lowest y
   // values at the top, and we have it the other way around ( i know i
   // shouldn't have done this.. :( ), we invert the value that the
   // scrollbar shows.  This is inverted again in
   // slotRightScrollValueChanged()...
-  mrightscroll->setValue( rightmin + ( rightmax - ( sr.bottom() / pw ) ) );
+  mrightscroll->setValue( (int) ( rightmin + ( rightmax - ( sr.bottom() / pw ) ) ) );
 
-  mbottomscroll->setMinValue( er.left() / pw );
-  mbottomscroll->setMaxValue( ( er.right() - sr.width() ) / pw );
-  mbottomscroll->setLineStep( sr.width() / pw / 10 );
-  mbottomscroll->setPageStep( sr.width() / pw / 1.2 );
-  mbottomscroll->setValue( sr.left() / pw );
+  mbottomscroll->setMinValue( (int)( er.left() / pw ) );
+  mbottomscroll->setMaxValue( (int)( ( er.right() - sr.width() ) / pw ) );
+  mbottomscroll->setLineStep( (int)( sr.width() / pw / 10 ) );
+  mbottomscroll->setPageStep( (int)( sr.width() / pw / 1.2 ) );
+  mbottomscroll->setValue( (int)( sr.left() / pw ) );
 
   mupdatingscrollbars = false;
 }
@@ -446,13 +450,28 @@ KigWidget* KigView::realWidget()
   return mrealwidget;
 }
 
-void KigView::slotExportToImage()
-{
-  ExportToImageDialog* d = new ExportToImageDialog( this, mdoc );
-  d->exec();
-}
-
 const KigDocument& KigWidget::document() const
 {
   return *mdocument;
 }
+
+QSize KigWidget::sizeHint() const
+{
+  return QSize( 600, 400 );
+}
+
+void KigWidget::wheelEvent( QWheelEvent* e )
+{
+  int delta = e->delta();
+  mview->scrollVertical( delta );
+}
+
+void KigView::scrollVertical( int delta )
+{
+  if ( delta >= 0 )
+    for ( int i = 0; i < delta; i += 120 )
+      mrightscroll->subtractLine();
+  else
+    for ( int i = 0; i >= delta; i -= 120 )
+      mrightscroll->addLine();
+}
