@@ -18,9 +18,11 @@
  USA
 **/
 
-
 #include "common.h"
 
+#include "../kig/kig_view.h"
+#include "../objects/object_imp.h"
+
 #include <cmath>
 
 #include <kdebug.h>
@@ -133,6 +135,10 @@ Coordinate calcIntersectionPoint( const LineData& l1, const LineData& l2 )
   return Coordinate( nx, ny );
 };
 
+void calcBorderPoints( Coordinate& p1, Coordinate& p2, const Rect& r )
+{
+  calcBorderPoints( p1.x, p1.y, p2.x, p2.y, r );
+};
 
 const LineData calcBorderPoints( const LineData& l, const Rect& r )
 {
@@ -395,3 +401,72 @@ double getDoubleFromUser( const QString& caption, const QString& label, double v
   if ( ok ) *ok = myok;
   return ret;
 };
+
+const Coordinate calcCenter(
+  const Coordinate& a, const Coordinate& b, const Coordinate& c )
+{
+  // this algorithm is written by my brother, Christophe Devriese
+  // <oelewapperke@ulyssis.org> ...
+  // I don't get it myself :)
+
+  double xdo = b.x-a.x;
+  double ydo = b.y-a.y;
+
+  double xao = c.x-a.x;
+  double yao = c.y-a.y;
+
+  double a2 = xdo*xdo + ydo*ydo;
+  double b2 = xao*xao + yao*yao;
+
+  double numerator = (xdo * yao - xao * ydo);
+  if ( numerator == 0 )
+  {
+    // problem:  xdo * yao == xao * ydo <=> xdo/ydo == xao / yao
+    // this means that the lines ac and ab have the same direction,
+    // which means they're the same line..
+    // FIXME: i would normally throw an error here, but KDE doesn't
+    // use exceptions, so i'm returning a bogus point :(
+    return (a+c)/2;
+  };
+  double denominator = 0.5 / numerator;
+
+  double centerx = a.x - (ydo * b2 - yao * a2) * denominator;
+  double centery = a.y + (xdo * b2 - xao * a2) * denominator;
+
+  return Coordinate(centerx, centery);
+}
+
+bool lineInRect( const Rect& r, const Coordinate& a, const Coordinate& b,
+                 const int width, const ObjectImp* imp, const KigWidget& w )
+{
+  double miss = w.screenInfo().normalMiss( width );
+  if ( fabs( a.x - b.x ) <= 1e-7 )
+  {
+    // too small to be useful..
+    return r.contains( Coordinate( a.x, r.center().y ), miss );
+  }
+  Coordinate dir = b - a;
+  double m = dir.y / dir.x;
+  double lefty = a.y + m * ( r.left() - a.x );
+  double righty = a.y + m * ( r.right() - a.x );
+  double minv = dir.x / dir.y;
+  double bottomx = a.x + minv * ( r.bottom() - a.y );
+  double topx = a.x + minv * ( r.top() - a.y );
+
+  // these are the intersections between the line, and the lines
+  // defined by the sides of the rectangle.
+  Coordinate leftint( r.left(), lefty );
+  Coordinate rightint( r.right(), righty );
+  Coordinate bottomint( bottomx, r.bottom() );
+  Coordinate topint( topx, r.top() );
+
+  // For each intersection, we now check if we contain the
+  // intersection ( this might not be the case for a segment, when the
+  // intersection is not between the begin and end point.. ) and if
+  // the rect contains the intersection..  If it does, we have a winner..
+  return
+    ( imp->contains( leftint, width, w ) && r.contains( leftint, miss ) ) ||
+    ( imp->contains( rightint, width, w ) && r.contains( rightint, miss ) ) ||
+    ( imp->contains( bottomint, width, w ) && r.contains( bottomint, miss ) ) ||
+    ( imp->contains( topint, width, w ) && r.contains( topint, miss ) );
+};
