@@ -1,6 +1,6 @@
 /**
    This file is part of Kig, a KDE program for Interactive Geometry...
-   Copyright (C) 2002  Dominique Devriese <devriese@kde.org>
+   Copyright (C) 2002-2003  Dominique Devriese <devriese@kde.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,22 +22,30 @@
 
 #include "../kig/kig_view.h"
 #include "../objects/object.h"
-#include "../objects/conic.h"
-#include "../objects/cubic.h"
+#include "../objects/curve_imp.h"
+#include "../objects/point_imp.h"
+#include "object_hierarchy.h"
 #include "common.h"
+#include "conic-common.h"
+#include "cubic-common.h"
 #include "coordinate_system.h"
 
 #include <qpen.h>
 
 #include <cmath>
 #include <stack>
+#include <functional>
+#include <algorithm>
 
-KigPainter::KigPainter( const ScreenInfo& si, QPaintDevice* device, bool no )
+KigPainter::KigPainter( const ScreenInfo& si, QPaintDevice* device,
+                        const KigDocument& doc, bool no )
   : mP ( device ),
+    mdoc( doc ),
     msi( si ),
     mNeedOverlay( no ),
     overlayenlarge( 0 )
 {
+  mP.setBackgroundColor( Qt::white );
 }
 
 KigPainter::~KigPainter()
@@ -76,8 +84,11 @@ void KigPainter::drawSegment( const Coordinate& from, const Coordinate& to )
   if( mNeedOverlay ) segmentOverlay( from, to );
 }
 
-void KigPainter::drawFatPoint( const Coordinate& p, double radius )
+void KigPainter::drawFatPoint( const Coordinate& p )
 {
+  int twidth = width == -1 ? 5 : width;
+  mP.setPen( QPen( color, 1, style ) );
+  double radius = twidth * pixelWidth();
   setBrushStyle( Qt::SolidPattern );
   Coordinate rad( radius, radius );
   rad /= 2;
@@ -87,6 +98,7 @@ void KigPainter::drawFatPoint( const Coordinate& p, double radius )
   QRect qr = toScreen( r );
   mP.drawEllipse( qr );
   if( mNeedOverlay ) mOverlay.push_back( qr );
+  mP.setPen( QPen( color, twidth, style ) );
 }
 
 void KigPainter::drawPoint( const Coordinate& p )
@@ -123,20 +135,20 @@ const Rect KigPainter::boundingRect( const Rect& r, const QString s,
 void KigPainter::setColor( const QColor& c )
 {
   color = c;
-  mP.setPen( QPen( color, width, style ) );
+  mP.setPen( QPen( color, width == -1 ? 1 : width, style ) );
 }
 
 void KigPainter::setStyle( const PenStyle c )
 {
   style = c;
-  mP.setPen( QPen( color, width, style ) );
+  mP.setPen( QPen( color, width == -1 ? 1 : width, style ) );
 }
 
-void KigPainter::setWidth( const uint c )
+void KigPainter::setWidth( const int c )
 {
   width = c;
   if (c > 0) overlayenlarge = c - 1;
-  mP.setPen( QPen( color, width, style ) );
+  mP.setPen( QPen( color, width == -1 ? 1 : width, style ) );
 }
 
 void KigPainter::setPen( const QPen& p )
@@ -166,21 +178,31 @@ void KigPainter::setBrushColor( const QColor& c )
   mP.setBrush( QBrush( brushColor, brushStyle ) );
 }
 
-void KigPainter::drawPolygon( const std::vector<Coordinate>& pts, bool winding, int index, int npoints )
+static void setContains( QRect& r, const QPoint& p )
 {
-  Rect sr;
+  if ( r.left() > p.x() ) r.setLeft( p.x() );
+  if ( r.right() < p.x() ) r.setRight( p.x() );
+  // this is correct, i think.  In qt the bottom has the highest y
+  // coord...
+  if ( r.bottom() < p.y() ) r.setBottom( p.x() );
+  if ( r.top() < p.y() ) r.setTop( p.x() );
+};
+
+void KigPainter::drawPolygon( const std::vector<QPoint>& pts,
+                              bool winding, int index, int npoints )
+{
+  QRect sr;
   // i know this isn't really fast, but i blame it all on Qt with its
   // stupid container classes... what's wrong with the STL ?
   QPointArray t( pts.size() );
   int c = 0;
-  for( std::vector<Coordinate>::const_iterator i = pts.begin(); i != pts.end(); ++i )
+  for( std::vector<QPoint>::const_iterator i = pts.begin(); i != pts.end(); ++i )
   {
-    sr.setContains( *i );
-    QPoint tt = toScreen (*i);
-    t.putPoints( c++, 1, tt.x(), tt.y() );
+    setContains( sr, *i );
+    t.putPoints( c++, 1, i->x(), i->y() );
   };
   mP.drawPolygon( t, winding, index, npoints );
-  mOverlay.push_back( toScreen( sr ) );
+  mOverlay.push_back( sr );
 }
 
 Rect KigPainter::window()
@@ -336,15 +358,15 @@ QPoint KigPainter::toScreen( const Coordinate p ) const
   return msi.toScreen( p );
 }
 
-void KigPainter::drawGrid( const CoordinateSystem& c )
+void KigPainter::drawGrid( const CoordinateSystem& c, bool showGrid, bool showAxes )
 {
-  c.drawGrid( *this );
+  c.drawGrid( *this, showGrid, showAxes );
   setWholeWinOverlay();
 }
 
 void KigPainter::drawObject( const Object* o, bool ss )
 {
-  o->drawWrap( *this, ss );
+  o->draw( *this, ss );
 }
 
 void KigPainter::drawObjects( const Objects& os )
@@ -437,21 +459,6 @@ void KigPainter::drawRay( const Coordinate& a, const Coordinate& b )
   drawSegment( a, tb );
 }
 
-void KigPainter::drawAngle( const Rect& surroundingRect, int startAngle, int angle )
-{
-  mP.drawArc( toScreen( surroundingRect ), startAngle, angle );
-  Rect r = surroundingRect;
-  r *= 1.2;
-  r.setCenter( surroundingRect.center() );
-//  if ( mNeedOverlay ) mOverlay.push_back( toScreen( r ) );
-  setWholeWinOverlay();
-}
-
-void KigPainter::drawFatPoint( const Coordinate& p )
-{
-  drawFatPoint( p, 5 * pixelWidth() );
-}
-
 inline Coordinate conicGetCoord( double theta, double ecostheta0,
                                  double esintheta0, double pdimen,
                                  const Coordinate& focus1 )
@@ -474,9 +481,8 @@ struct workitem
   coordparampair second;
   Rect		 *overlay;
 };
-//typedef std::pair<coordparampair, coordparampair> workitem;
 
-void KigPainter::drawConic( const ConicPolarEquationData& data )
+void KigPainter::drawConic( const ConicPolarData& data )
 {
   // @author Maurizio Paolini wrote an original version of this, ( the
   // math in conicGetCoord() is still his ), I ( Dominique ) adapted
@@ -509,10 +515,10 @@ void KigPainter::drawConic( const ConicPolarEquationData& data )
   // the strategy for generating the overlay structure is the same
   // recursive-like used to draw the segments: a new rectangle is
   // generated whenever the length of a segment becomes lower than
-  // overlayRectSize(), or if the segment would be draw anyway
-  // to avoid strange things happening we impose that the distance
-  // in parameter space be less than a threshold before generating
-  // any overlay.
+  // overlayRectSize(), or if the segment would be drawn anyway
+  // to avoid strange things from happening we impose that the
+  // distance in parameter space be less than a threshold before
+  // generating any overlay.
   //
   // The third parameter in workitem is a pointer into a stack of
   // all generated rectangles (in real coordinate space); if 0
@@ -630,13 +636,13 @@ void KigPainter::drawConic( const ConicPolarEquationData& data )
 }
 
 void KigPainter::drawCubicRecurse (
-                   double& xleft, double& yleft, bool& validleft,
-                   int& numrootsleft,
-                   double &xright, double &yright, bool &validright,
-                   int &numrootsright,
-                   const CubicCartesianEquationData &data, int &root,
-                   double &ymin, double &ymax, double &tol,
-                   bool& tNeedOverlay, Rect& overlay)
+  double& xleft, double& yleft, bool& validleft,
+  int& numrootsleft,
+  double &xright, double &yright, bool &validright,
+  int &numrootsright,
+  const CubicCartesianData &data, int &root,
+  double &ymin, double &ymax, double &tol,
+  bool& tNeedOverlay, Rect& overlay)
 {
   double ltol = tol;
   if ( ! ( validleft && validright && numrootsleft == numrootsright ) )
@@ -658,7 +664,7 @@ void KigPainter::drawCubicRecurse (
     bool validmiddle;
     int numrootsmiddle;
     double ymiddle = calcCubicYvalue ( xmiddle, ymin, ymax, root, data,
-                   validmiddle, numrootsmiddle );
+                                       validmiddle, numrootsmiddle );
     Coordinate pmiddle = Coordinate( xmiddle, ymiddle );
     if ( validmiddle && tNeedOverlay ) overlay.setContains( pmiddle );
     drawCubicRecurse ( xleft, yleft, validleft, numrootsleft,
@@ -670,7 +676,7 @@ void KigPainter::drawCubicRecurse (
   }
 }
 
-void KigPainter::drawCubic( const CubicCartesianEquationData& data )
+void KigPainter::drawCubic( const CubicCartesianData& data )
 {
   // we manage our own overlay
   bool tNeedOverlay = mNeedOverlay;
@@ -713,10 +719,285 @@ void KigPainter::drawCubic( const CubicCartesianEquationData& data )
 void KigPainter::drawLine( const LineData& d )
 {
   LineData l = calcBorderPoints( d, window() );
-  drawSegment( l );
+  drawSegment( l.a, l.b );
 }
 
 void KigPainter::drawSegment( const LineData& d )
 {
   drawSegment( d.a, d.b );
 }
+
+void KigPainter::drawRay( const LineData& d )
+{
+  drawRay( d.a, d.b );
+}
+
+void KigPainter::drawAngle( const Coordinate& cpoint, const double dstartangle,
+                            const double dangle )
+{
+  // convert to 16th of degrees...
+  const int startangle = static_cast<int>( 16*180*dstartangle / M_PI );
+  const int angle = static_cast<int>( 16*180*dangle / M_PI );
+
+  QPoint point = toScreen( cpoint );
+
+//   int radius = mP.window().width() / 5;
+  int radius = 50;
+  QRect surroundingRect( 0, 0, radius*2, radius*2 );
+  surroundingRect.moveCenter( point );
+
+  mP.drawArc( surroundingRect, startangle, angle );
+
+  // now for the arrow...
+  QPoint end( static_cast<int>( point.x() + radius * cos( dstartangle + dangle ) ),
+              static_cast<int>( point.y() - radius * sin( dstartangle + dangle ) ) );
+  QPoint vect = (end - point);
+  double vectlen = sqrt( vect.x() * vect.x() + vect.y() * vect.y() );
+  QPoint orthvect( -vect.y(), vect.x() );
+  vect = vect * 6 / vectlen;
+  orthvect = orthvect * 6 / vectlen;
+
+  std::vector<QPoint> arrow;
+  arrow.push_back( end );
+  arrow.push_back( end + orthvect + vect );
+  arrow.push_back( end + orthvect - vect );
+
+  setBrushStyle( Qt::SolidPattern );
+  drawPolygon( arrow );
+
+//  if ( mNeedOverlay ) mOverlay.push_back( toScreen( r ) );
+  setWholeWinOverlay();
+}
+
+void KigPainter::drawPolygon( const std::vector<Coordinate>& pts,
+                              bool winding, int index, int npoints )
+{
+  using namespace std;
+  vector<QPoint> points;
+  for ( uint i = 0; i < pts.size(); ++i )
+    points.push_back( toScreen( pts[i] ) );
+  drawPolygon( points, winding, index, npoints );
+}
+
+void KigPainter::drawVector( const Coordinate& a, const Coordinate& b )
+{
+  // bugfix...
+  if ( a == b ) return;
+  // the segment
+  drawSegment( a, b );
+  // the arrows...
+  Coordinate dir = b - a;
+  Coordinate perp( dir.y, -dir.x );
+  double length = perp.length();
+  perp *= 10* pixelWidth();
+  perp /= length;
+  dir *= 10 * pixelWidth();
+  dir /= length;
+  Coordinate c = b - dir + perp;
+  Coordinate d = b - dir - perp;
+  drawSegment( b, c );
+  drawSegment( b, d );
+}
+
+inline Coordinate locusGetCoord( double p, const CurveImp* curve, const ObjectHierarchy& h,
+                                 bool& valid, const KigDocument& doc )
+{
+  Coordinate pt = curve->getPoint( p, valid, doc );
+  if ( ! valid ) return Coordinate();
+  PointImp pimp( pt );
+  Args args;
+  args.push_back( &pimp );
+  std::vector<ObjectImp*> calced = h.calc( args, doc );
+  assert( calced.size() == 1 );
+  ObjectImp* o = calced.front();
+  Coordinate ret;
+  if ( o->inherits( ObjectImp::ID_PointImp ) )
+  {
+    valid = true;
+    ret = static_cast<PointImp*>( o )->coordinate();
+  }
+  else
+    valid = false;
+  delete o;
+  return ret;
+};
+
+void KigPainter::drawLocus( const CurveImp* curve, const ObjectHierarchy& hier )
+{
+  /// this function is based on drawConic, and its code comes mostly
+  /// from there too..  ( drawConic was based on an old version of
+  /// drawLocus, actually, but that's ancient history by now :) )
+
+  // we manage our own overlay
+  bool tNeedOverlay = mNeedOverlay;
+  mNeedOverlay = false;
+
+  QPen pen = mP.pen();
+  pen.setCapStyle( Qt::RoundCap );
+  mP.setPen( pen );
+
+  // this stack contains pairs of Coordinates that we still need to
+  // process:
+  std::stack<workitem> workstack;
+  // mp: this stack contains all the generated overlays:
+  // the strategy for generating the overlay structure is the same
+  // recursive-like used to draw the segments: a new rectangle is
+  // generated whenever the length of a segment becomes lower than
+  // overlayRectSize(), or if the segment would be drawn anyway
+  // to avoid strange things from happening we impose that the distance
+  // in parameter space be less than a threshold before generating
+  // any overlay.
+  //
+  // The third parameter in workitem is a pointer into a stack of
+  // all generated rectangles (in real coordinate space); if 0
+  // there is no rectangles associated to that segment yet.
+  //
+  // Using the final mOverlay stack would be much more efficient, but
+  // 1. needs transformations into window space
+  // 2. would be more difficult to drop rectangles not intersecting
+  //    the window.
+  std::stack<Rect> overlaystack;
+
+  bool valid = true; ///
+
+  // mp: the original version in which an initial set of 20 intervals
+  // were pushed onto the stack is replaced by a single interval and
+  // by forcing subdivision till h < hmax (with more or less the same
+  // final result).
+  // First push the [0,1] interval into the stack:
+  workstack.push( workitem(
+                    coordparampair( 0, locusGetCoord( 0, curve, hier, valid, mdoc ) ), ///
+                    coordparampair( 1, locusGetCoord( 0, curve, hier, valid, mdoc ) ), ///
+                    0 ) );
+
+  // maxlength is the square of the maximum size that we allow
+  // between two points..
+  double maxlength = 1.5 * pixelWidth();
+  maxlength *= maxlength;
+  // error squared is required to be less that sigma (half pixel)
+  double sigma = maxlength/4;
+  // distance between two parameter values cannot be too small
+  double hmin = 1e-4;
+  // distance between two parameter values cannot be too large
+  double hmax = M_PI/20; ///
+  double hmaxoverlay = M_PI/4; ///
+
+  int count = 1;               // the number of segments we've already
+                               // visited...
+  static const int maxnumberofpoints = 1000;
+
+  const Rect& sr = window();
+
+  // we don't use recursion, but a stack based approach for efficiency
+  // concerns...
+  while ( ! workstack.empty() && count < maxnumberofpoints )
+  {
+    workitem curitem = workstack.top();
+    workstack.pop();
+    bool curitemok = true;
+    while ( curitemok && count++ < maxnumberofpoints )
+    {
+      // we take the middle parameter of the two previous points...
+      Coordinate p0 = curitem.first.second;
+      Coordinate p1 = curitem.second.second;
+      double t2 = ( curitem.first.first + curitem.second.first ) / 2;
+      double h = fabs( curitem.second.first - curitem.first.first ) /2;
+      Rect *overlaypt = curitem.overlay;
+      Coordinate p2 = locusGetCoord( t2, curve, hier, valid, mdoc ); ///
+
+      bool dooverlay = ! overlaypt && h < hmaxoverlay
+ && fabs( p0.x - p1.x ) <= overlayRectSize()
+ && fabs( p0.y - p1.y ) <= overlayRectSize();
+      bool addn = valid && sr.contains( p2 ); ///
+      // estimated error between the curve and the segments
+      double errsq = (0.5*p0 + 0.5*p1 - p2).squareLength();
+      errsq /= 4;
+      curitemok = false;
+      bool dodraw = valid && h < hmax && ( errsq < sigma || h < hmin ); ///
+      if ( tNeedOverlay && ( dooverlay || dodraw ) )
+      {
+        Rect newoverlay( p0, p1 );
+        overlaystack.push( newoverlay );
+        overlaypt = &overlaystack.top();
+      }
+      if ( overlaypt ) overlaypt->setContains( p2 );
+      if ( dodraw )
+      {
+        // draw the two segments
+        QPoint tp0 = toScreen(p0);
+        QPoint tp1 = toScreen(p1);
+        QPoint tp2 = toScreen(p2);
+        mP.drawLine( tp0, tp2 );
+        mP.drawLine( tp2, tp1 );
+      }
+      else if ( valid ) ///
+      {
+        // push into stack in order to process both subintervals
+        if (addn || sr.contains( p0 ) || h >= hmax)
+          workstack.push( workitem( curitem.first, coordparampair( t2, p2 ),
+                                    overlaypt ) );
+        if (addn || sr.contains( p1 ) || h >= hmax)
+        {
+          curitem = workitem( coordparampair( t2, p2 ), curitem.second ,
+                              overlaypt );
+          curitemok = true;
+        }
+      }
+    }
+  }
+
+  assert ( tNeedOverlay || overlaystack.empty() );
+  if ( tNeedOverlay )
+  {
+    Rect border = window();
+    while ( ! overlaystack.empty() )
+    {
+      Rect overlay = overlaystack.top();
+      overlaystack.pop();
+      if (overlay.intersects( border ))
+        mOverlay.push_back( toScreenEnlarge( overlay ) );
+    }
+  }
+  mNeedOverlay = tNeedOverlay;
+  pen.setCapStyle( Qt::FlatCap );
+  mP.setPen( pen );
+}
+
+void KigPainter::drawTextFrame( const Rect& frame,
+                                const QString& s, bool needframe )
+{
+  QPen oldpen = mP.pen();
+  QBrush oldbrush = mP.brush();
+  if ( needframe )
+  {
+    // inspired upon kgeo, thanks to Marc Bartsch, i've taken some of
+    // his code too..
+    setPen( QPen( Qt::black, 1 ) );
+    setBrush( QBrush( QColor( 255, 255, 222 ) ) );
+    drawRect( frame );
+    setPen( QPen( QColor( 197, 194, 197 ), 1, Qt::SolidLine ) );
+
+    QRect qr = toScreen( frame );
+
+    mP.drawLine( qr.topLeft(), qr.topRight() );
+    mP.drawLine( qr.topLeft(), qr.bottomLeft() );
+  };
+  setPen( oldpen );
+  setBrush( oldbrush );
+  drawText( frame, s, Qt::AlignVCenter | Qt::AlignHCenter );
+}
+
+void KigPainter::drawArc( const Coordinate& center, const double radius,
+                          const double dstartangle, const double dangle )
+{
+  // convert to 16th of degrees...
+  const int startangle = static_cast<int>( 16*180*dstartangle / M_PI );
+  const int angle = static_cast<int>( 16*180*dangle / M_PI );
+
+  Rect krect( 0, 0, 2*radius, 2*radius );
+  krect.setCenter( center );
+  QRect rect = toScreen( krect );
+
+  mP.drawArc( rect, startangle, angle );
+  setWholeWinOverlay();
+}
