@@ -5,7 +5,7 @@
 
 OLDPWD=$(pwd)
 
-VERSION="0.3.1"
+VERSION="0.4.1"
 NAME="kig"
 
 TEMPDIR="/tmp/$NAME-package-temp"
@@ -18,7 +18,7 @@ cd $TEMPDIR
 	--name "$NAME" \
 	--version "$VERSION" \
 	--log="$TEMPDIR/log" \
-	~/src/test/kdeedu kig
+	~/src/package/kdeedu kig
 
 cd $OLDPWD
 
@@ -27,5 +27,3 @@ echo "Don't forget to change #MIN_CONFIG in "
 echo "configure.in.in to "
 echo "#MIN_CONFIG( 3.0 ) on packaging, so kig "
 echo "won't depend on Qt 3.1 but 3.0."
-echo "update: also use an admin dir and Makefile.cvs "
-echo "from kde 3.0"
